# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
# from horizon import messages
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

# from openstack_dashboard.api import iotronic
from openstack_dashboard import api
from openstack_dashboard import policy

from iotronic_ui.iot.boards import forms as project_forms
from iotronic_ui.iot.boards import tables as project_tables
from iotronic_ui.iot.boards import tabs as project_tabs


LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = project_tables.BoardsTable
    template_name = 'iot/boards/index.html'
    page_title = _("Boards")

    def get_data(self):
        boards = []

        # Admin
        if policy.check((("iot", "iot:list_all_boards"),), self.request):
            try:
                boards = api.iotronic.board_list(self.request, None, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve boards list.'))

        # Admin_iot_project
        elif policy.check((("iot", "iot:list_project_boards"),), self.request):
            try:
                boards = api.iotronic.board_list(self.request, None, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user boards list.'))

        # Other users
        else:
            try:
                boards = api.iotronic.board_list(self.request, None, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user boards list.'))

        for board in boards:
            board_services = api.iotronic.services_on_board(self.request,
                                                            board.uuid,
                                                            True)

            # TO BE REMOVED
            # We are filtering the services that starts with "webservice"
            # ------------------------------------------------------------
            filter_ws = []
            for service in board_services:
                if ((service["name"] != "webservice") and 
                   (service["name"] != "webservice_ssl")):
                    filter_ws.append(service)

            board_services = filter_ws
            # ------------------------------------------------------------

            # board.__dict__.update(dict(services=board_services))
            board._info.update(dict(services=board_services))

            if board.fleet != None:            
                fleet_info = api.iotronic.fleet_get(self.request,
                                                    board.fleet,
                                                    None)

                board.fleet_name = fleet_info.name
            else:
                board.fleet_name = None

        boards.sort(key=lambda b: b.name)
        return boards


class CreateView(forms.ModalFormView):
    template_name = 'iot/boards/create.html'
    modal_header = _("Create Board")
    form_id = "create_board_form"
    form_class = project_forms.CreateBoardForm
    submit_label = _("Create Board")
    submit_url = reverse_lazy("horizon:iot:boards:create")
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Create Board")


class UpdateView(forms.ModalFormView):
    template_name = 'iot/boards/update.html'
    modal_header = _("Update Board")
    form_id = "update_board_form"
    form_class = project_forms.UpdateBoardForm
    submit_label = _("Update Board")
    submit_url = "horizon:iot:boards:update"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Update Board")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)
        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()
        location = board.location[0]

        return {'uuid': board.uuid,
                'name': board.name,
                'mobile': board.mobile,
                'owner': board.owner,
                'fleet_id': board.fleet,
                'latitude': location["latitude"],
                'longitude': location["longitude"],
                'altitude': location["altitude"]}


class EnableServiceView(forms.ModalFormView):
    template_name = 'iot/boards/enableservice.html'
    modal_header = _("Enable Service(s)")
    form_id = "service_enable_form"
    form_class = project_forms.EnableServiceForm
    submit_label = _("Enable")
    # submit_url = reverse_lazy("horizon:iot:boards:enableservice")
    submit_url = "horizon:iot:boards:enableservice"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Action")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)

        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(EnableServiceView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        # Populate available services
        cloud_services = api.iotronic.service_list(self.request, None)
        board_services = api.iotronic.services_on_board(self.request,
                                                        board.uuid,
                                                        True)
        cloud_services.sort(key=lambda b: b.name)
        service_list = []
        # LOG.debug('CLOUD SERVICES: %s', cloud_services)

        for cloud_service in cloud_services:

            uuid = cloud_service._info["uuid"]
            name = cloud_service._info["name"]

            if len(board_services) == 0:

                # BEFORE filtering necessity
                # service_list.append((cloud_service._info["uuid"],
                #                     _(cloud_service._info["name"])))

                # AFTER filtering necessity
                # We are filtering the services that starts with "webservice"
                if ((name != "webservice") and (name != "webservice_ssl")):
                    service_list.append((uuid, _(name)))

            else:
                counter = 0
                for board_service in board_services:
                    if board_service["uuid"] == cloud_service._info["uuid"]:
                        break
                    elif counter != len(board_services) - 1:
                        counter += 1
                    else:
                        # BEFORE filtering necessity
                        # service_list.append((uuid, _(name)))

                        # AFTER filtering necessity
                        # We are filtering the services that starts
                        # with "webservice"
                        if ((name != "webservice") and
                           ("name" != "webservice_ssl")):
                            service_list.append((uuid, _(name)))

        # LOG.debug('SERVICES: %s', service_list)
        return {'uuid': board.uuid,
                'name': board.name,
                'service_list': service_list}


class DisableServiceView(forms.ModalFormView):
    template_name = 'iot/boards/disableservice.html'
    modal_header = _("Disable Service(s)")
    form_id = "service_disable_form"
    form_class = project_forms.DisableServiceForm
    submit_label = _("Disable")
    # submit_url = reverse_lazy("horizon:iot:boards:disableservice")
    submit_url = "horizon:iot:boards:disableservice"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Action")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)

        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(DisableServiceView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        # Populate available services
        cloud_services = api.iotronic.service_list(self.request, None)
        board_services = api.iotronic.services_on_board(self.request,
                                                        board.uuid,
                                                        True)
        cloud_services.sort(key=lambda b: b.name)
        service_list = []

        # BEFORE filtering necessity
        """
        for cloud_service in cloud_services:
            for board_service in board_services:
                if board_service["uuid"] == cloud_service._info["uuid"]:
                    service_list.append((cloud_service._info["uuid"],
                                        _(cloud_service._info["name"])))
        """

        # AFTER filtering necessity
        # We are filtering the services that starts with "webservice"
        # ------------------------------------------------------------
        for cloud_service in cloud_services:
            for board_service in board_services:
                if ((board_service["uuid"] == cloud_service._info["uuid"]) and
                   ((board_service["name"] != "webservice") and
                   (board_service["name"] != "webservice_ssl"))): 
                    service_list.append((cloud_service._info["uuid"],
                                        _(cloud_service._info["name"])))
        # ------------------------------------------------------------

        return {'uuid': board.uuid,
                'name': board.name,
                'service_list': service_list}


class AttachPortView(forms.ModalFormView):
    template_name = 'iot/boards/attachport.html'
    modal_header = _("Attach")
    form_id = "attach_boardport_form"
    form_class = project_forms.AttachPortForm
    submit_label = _("Attach")
    # submit_url = reverse_lazy("horizon:iot:boards:attachport")
    submit_url = "horizon:iot:boards:attachport"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Attach port")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)
        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(AttachPortView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        # Populate networks
        networks = api.neutron.network_list(self.request)
        net_choices = []

        networks.sort(key=lambda b: b.name)
        for net in networks:
            for subnet in net["subnets"]:
                net_choices.append((net["id"] + ':' + subnet["id"],
                                   _(net["name"] + ':' + subnet["name"])))

        return {'uuid': board.uuid,
                'name': board.name,
                'networks_list': net_choices}


class DetachPortView(forms.ModalFormView):
    template_name = 'iot/boards/detachport.html'
    modal_header = _("Detach")
    form_id = "detach_boardport_form"
    form_class = project_forms.DetachPortForm
    submit_label = _("Detach")
    # submit_url = reverse_lazy("horizon:iot:boards:detachport")
    submit_url = "horizon:iot:boards:detachport"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Detach port")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)
        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(DetachPortView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        ports = api.iotronic.port_list(self.request, board.uuid)

        # TO BE REMOVED (change it once the port_list per board is
        # completed and tested !
        # ################################################################
        # LOG.debug("PORTS: %s", ports)

        ports.sort(key=lambda b: b.name)
        filtered_ports = []
        for port in ports:
            if port._info["board_uuid"] == board.uuid:
                filtered_ports.append((port._info["uuid"],
                                      _(port._info["ip"])))

        ports = filtered_ports
        # ################################################################

        # Populate board ports
        return {'uuid': board.uuid,
                'name': board.name,
                'ports': ports}


class EnableWebServiceView(forms.ModalFormView):
    template_name = 'iot/boards/enablewebservice.html'
    modal_header = _("Enable Web Services Manager")
    form_id = "webservice_enable_form"
    form_class = project_forms.EnableWebServiceForm
    submit_label = _("Enable")
    # submit_url = reverse_lazy("horizon:iot:boards:enablewebservice")
    submit_url = "horizon:iot:boards:enablewebservice"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Enable")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)

        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(EnableWebServiceView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        return {'uuid': board.uuid,
                'name': board.name}


class DisableWebServiceView(forms.ModalFormView):
    template_name = 'iot/boards/disablewebservice.html'
    modal_header = _("Disable Web Services Manager")
    form_id = "webservice_disable_form"
    form_class = project_forms.DisableWebServiceForm
    submit_label = _("Disable")
    # submit_url = reverse_lazy("horizon:iot:boards:disablewebservice")
    submit_url = "horizon:iot:boards:disablewebservice"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Disable")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)

        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(DisableWebServiceView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        return {'uuid': board.uuid,
                'name': board.name}


class RemovePluginsView(forms.ModalFormView):
    template_name = 'iot/boards/removeplugins.html'
    modal_header = _("Remove Plugins from board")
    form_id = "remove_boardplugins_form"
    form_class = project_forms.RemovePluginsForm
    submit_label = _("Remove")
    # submit_url = reverse_lazy("horizon:iot:boards:removeplugins")
    submit_url = "horizon:iot:boards:removeplugins"
    success_url = reverse_lazy('horizon:iot:boards:index')
    page_title = _("Remove Plugins from board")

    @memoized.memoized_method
    def get_object(self):
        try:
            return api.iotronic.board_get(self.request,
                                          self.kwargs['board_id'],
                                          None)
        except Exception:
            redirect = reverse("horizon:iot:boards:index")
            exceptions.handle(self.request,
                              _('Unable to get board information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(RemovePluginsView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        # Populate plugins
        # TO BE DONE.....filter by available on this board!!!
        # plugins = api.iotronic.plugin_list(self.request, None, None)
        plugins = api.iotronic.plugins_on_board(self.request, board.uuid)

        plugins.sort(key=lambda b: b["name"])

        plugin_list = []
        for plugin in plugins:
            plugin_list.append((plugin["id"], _(plugin["name"])))

        return {'uuid': board.uuid,
                'name': board.name,
                'plugin_list': plugin_list}


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.BoardDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ board.name|default:board.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        board = self.get_data()
        context["board"] = board
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(board)

        return context

    def _get_actions(self, board):
        table = project_tables.BoardsTable(self.request)
        return table.render_row_actions(board)

    @memoized.memoized_method
    def get_data(self):

        board = []

        board_id = self.kwargs['board_id']
        try:

            board_ports = []

            board = api.iotronic.board_get(self.request, board_id, None)

            # FIX this problem with the new APIs
            # (remove the "if" clause with a better approach)
            # #################################################################
            ports = api.iotronic.port_list(self.request, board_id)

            for port in ports:
                if port._info["board_uuid"] == board_id:
                    board_ports.append(port._info)
            board._info.update(dict(ports=board_ports))
            # #################################################################

            board_services = api.iotronic.services_on_board(self.request,
                                                            board_id, True)
            board._info.update(dict(services=board_services))

            board_plugins = api.iotronic.plugins_on_board(self.request,
                                                          board_id)
            board._info.update(dict(plugins=board_plugins))

            board_webservices = api.iotronic.webservices_on_board(self.request,
                                                                  board_id)
            board._info.update(dict(webservices=board_webservices))

            # Adding fleet name
            if board.fleet != None:
                fleet_info = api.iotronic.fleet_get(self.request,
                                                    board.fleet,
                                                    None)

                board.fleet_name = fleet_info.name
            else:
                board.fleet_name = None

            # LOG.debug("BOARD: %s\n\n%s", board, board._info)

        except Exception:
            msg = ('Unable to retrieve board %s information') % {'name':
                                                                 board.name}
            exceptions.handle(self.request, msg, ignore=True)
        return board

    def get_tabs(self, request, *args, **kwargs):
        board = self.get_data()
        return self.tab_group_class(request, board=board, **kwargs)


class BoardDetailView(DetailView):
    redirect_url = 'horizon:iot:boards:index'

    def _get_actions(self, board):
        table = project_tables.BoardsTable(self.request)
        return table.render_row_actions(board)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
# from horizon import messages
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

from iotronic_ui.iot.services import forms as project_forms
from iotronic_ui.iot.services import tables as project_tables
from iotronic_ui.iot.services import tabs as project_tabs


LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = project_tables.ServicesTable
    template_name = 'iot/services/index.html'
    page_title = _("Services")

    def get_data(self):
        services = []

        # Admin
        if policy.check((("iot", "iot:list_all_services"),), self.request):
            try:
                services = iotronic.service_list(self.request, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve services list.'))

        # Admin_iot_project
        elif policy.check((("iot", "iot:list_project_services"),),
                          self.request):
            try:
                services = iotronic.service_list(self.request, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user services list.'))

        # Other users
        else:
            try:
                services = iotronic.service_list(self.request, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user services list.'))

        return services


class CreateView(forms.ModalFormView):
    template_name = 'iot/services/create.html'
    modal_header = _("Create Service")
    form_id = "create_service_form"
    form_class = project_forms.CreateServiceForm
    submit_label = _("Create Service")
    submit_url = reverse_lazy("horizon:iot:services:create")
    success_url = reverse_lazy('horizon:iot:services:index')
    page_title = _("Create Service")


class UpdateView(forms.ModalFormView):
    template_name = 'iot/services/update.html'
    modal_header = _("Update Service")
    form_id = "update_service_form"
    form_class = project_forms.UpdateServiceForm
    submit_label = _("Update Service")
    submit_url = "horizon:iot:services:update"
    success_url = reverse_lazy('horizon:iot:services:index')
    page_title = _("Update Service")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.service_get(self.request,
                                        self.kwargs['service_id'],
                                        None)
        except Exception:
            redirect = reverse("horizon:iot:services:index")
            exceptions.handle(self.request,
                              _('Unable to get service information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        service = self.get_object()

        return {'uuid': service.uuid,
                'name': service.name,
                'port': service.port,
                'protocol': service.protocol}


class ActionView(forms.ModalFormView):
    template_name = 'iot/services/action.html'
    modal_header = _("Service Action")
    form_id = "service_action_form"
    form_class = project_forms.ServiceActionForm
    submit_label = _("Service Action")
    # submit_url = reverse_lazy("horizon:iot:services:action")
    submit_url = "horizon:iot:services:action"
    success_url = reverse_lazy('horizon:iot:services:index')
    page_title = _("Service Action")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.service_get(self.request,
                                        self.kwargs['service_id'],
                                        None)
        except Exception:
            redirect = reverse("horizon:iot:services:index")
            exceptions.handle(self.request,
                              _('Unable to get service information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(ActionView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        service = self.get_object()

        # Populate boards
        boards = iotronic.board_list(self.request, "online", None, None)
        boards.sort(key=lambda b: b.name)

        board_list = []
        for board in boards:
            board_list.append((board.uuid, _(board.name)))

        return {'uuid': service.uuid,
                'name': service.name,
                'board_list': board_list}


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.ServiceDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ service.name|default:service.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        service = self.get_data()
        context["service"] = service
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(service)

        return context

    def _get_actions(self, service):
        table = project_tables.ServicesTable(self.request)
        return table.render_row_actions(service)

    @memoized.memoized_method
    def get_data(self):
        service_id = self.kwargs['service_id']
        try:
            service = iotronic.service_get(self.request, service_id, None)
        except Exception:
            s = service.name
            msg = ('Unable to retrieve service %s information') % {'name': s}
            exceptions.handle(self.request, msg, ignore=True)
        return service

    def get_tabs(self, request, *args, **kwargs):
        service = self.get_data()
        return self.tab_group_class(request, service=service, **kwargs)


class ServiceDetailView(DetailView):
    redirect_url = 'horizon:iot:services:index'

    def _get_actions(self, service):
        table = project_tables.ServicesTable(self.request)
        return table.render_row_actions(service)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
# from horizon import messages
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

from iotronic_ui.iot.webservices import forms as project_forms
from iotronic_ui.iot.webservices import tables as project_tables
from iotronic_ui.iot.webservices import tabs as project_tabs


LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = project_tables.WebservicesTable
    template_name = 'iot/webservices/index.html'
    page_title = _("Web Services")

    def get_data(self):
        webservices = []
        en_webservices = []

        # Admin
        if policy.check((("iot", "iot:list_all_webservices"),), self.request):
            try:
                webservices = iotronic.webservice_list(self.request,
                                                       None)
                en_webservices = iotronic.webservice_enabled_list(self.request)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve webservices list.'))

        # Admin_iot_project
        elif policy.check((("iot", "iot:list_project_webservices"),),
                          self.request):
            try:
                webservices = iotronic.webservice_list(self.request,
                                                       None)
                en_webservices = iotronic.webservice_enabled_list(self.request)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve webservices list.'))

        # Other users
        else:
            try:
                webservices = iotronic.webservice_list(self.request,
                                                       None)
                en_webservices = iotronic.webservice_enabled_list(self.request)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve webservices list.'))

        # Append some information to the webservice
        # LOG.debug('WSS: %s', webservices)
        for ws_en in en_webservices:

            ws_list = []

            for ws in webservices:
                if ws_en.board_uuid == ws.board_uuid:

                    service_url = "https://" + ws.name + "." + ws_en.dns +\
                                  "." + ws_en.zone
                    ws_list.append({"local_port": ws.port,
                                    "service_url": service_url})

                    ws_en.uuid = ws.uuid

            board = iotronic.board_get(self.request, ws_en.board_uuid, None)
            ws_en.name = board.name
            ws_en._info.update(dict(webservices=ws_list))

        return en_webservices


class ExposeView(forms.ModalFormView):
    template_name = 'iot/webservices/expose.html'
    modal_header = _("Expose Web Service")
    form_id = "expose_webservice_form"
    form_class = project_forms.ExposeWebserviceForm
    submit_label = _("Expose")
    submit_url = "horizon:iot:webservices:expose"
    success_url = reverse_lazy('horizon:iot:webservices:index')
    page_title = _("Expose Web Service")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.board_get(self.request,
                                      self.kwargs['board_id'],
                                      None)
        except Exception:
            redirect = reverse("horizon:iot:webservices:index")
            exceptions.handle(self.request,
                              _('Unable to get webservice information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(ExposeView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()
        board_exp_info = iotronic.webservice_get_enabled_info(self.request,
                                                              board.uuid)
        zone = board_exp_info.dns + "." + board_exp_info.zone

        return {'uuid': board.uuid, 'name': board.name, 'zone': zone}


class UnexposeView(forms.ModalFormView):
    template_name = 'iot/webservices/unexpose.html'
    modal_header = _("Unexpose Web Service")
    form_id = "unexpose_webservice_form"
    form_class = project_forms.UnexposeWebserviceForm
    submit_label = _("Unexpose")
    submit_url = "horizon:iot:webservices:unexpose"
    success_url = reverse_lazy('horizon:iot:webservices:index')
    page_title = _("Unexpose Web Service")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.board_get(self.request,
                                      self.kwargs['board_id'],
                                      None)
        except Exception:
            redirect = reverse("horizon:iot:webservices:index")
            exceptions.handle(self.request,
                              _('Unable to get webservice information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UnexposeView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        board = self.get_object()

        # Populate web services on board
        ws_onboard = iotronic.webservices_on_board(self.request, board.uuid)
        ws_onboard.sort(key=lambda b: b["name"])

        ws_onboard_list = []
        for ws in ws_onboard:
            ws_onboard_list.append((ws["uuid"], _(ws["name"])))

        return {'uuid': board.uuid,
                'name': board.name,
                'ws_onboard': ws_onboard_list}


"""
class DetailView(tabs.TabView):
    tab_group_class = project_tabs.WebserviceDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ webservice.name|default:webservice.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        webservice = self.get_data()
        context["webservice"] = webservice
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(webservice)

        return context

    def _get_actions(self, webservice):
        table = project_tables.WebservicesTable(self.request)
        return table.render_row_actions(webservice)

    @memoized.memoized_method
    def get_data(self):
        webservice = []

        webservice_id = self.kwargs['webservice_id']
        try:
            webservice = iotronic.webservice_get(self.request,
                                                 webservice_id,
                                                 None)
            board = iotronic.board_get(self.request,
                                       webservice.board_uuid,
                                       None)

            webservice._info.update({u'board_name': board.name})
            webservice.board_name = board.name
            # LOG.debug('WS: %s', webservice)

        except Exception:
            s = webservice.name
            msg = ('Unable to retrieve webservice %s information') % {'name': s}
            exceptions.handle(self.request, msg, ignore=True)

        return webservice

    def get_tabs(self, request, *args, **kwargs):
        webservice = self.get_data()
        return self.tab_group_class(request, webservice=webservice, **kwargs)


class WebserviceDetailView(DetailView):
    redirect_url = 'horizon:iot:webservices:index'

    def _get_actions(self, webservice):
        table = project_tables.WebservicesTable(self.request)
        return table.render_row_actions(webservice)
"""



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.conf.urls import url

from iotronic_ui.iot.webservices import views


urlpatterns = [
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^(?P<board_id>[^/]+)/expose/$',
        views.ExposeView.as_view(), name='expose'),
    url(r'^(?P<board_id>[^/]+)/unexpose/$',
        views.UnexposeView.as_view(), name='unexpose'),
    # url(r'^(?P<webservice_id>[^/]+)/detail/$', views.WebserviceDetailView.as_view(),
    #     name='detail'),
]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

LOG = logging.getLogger(__name__)


class CreateFleetForm(forms.SelfHandlingForm):
    name = forms.CharField(label=_("Fleet Name"))

    description = forms.CharField(
        label=_("Description"),
        widget=forms.Textarea(
            attrs={'class': 'switchable', 'data-slug': 'slug-description'})
    )

    def handle(self, request, data):
        try:
            iotronic.fleet_create(request, data["name"],
                                    data["description"])

            messages.success(request, _("Fleet " + str(data["name"]) +
                                        " created successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to create fleet.'))


class UpdateFleetForm(forms.SelfHandlingForm):
    uuid = forms.CharField(label=_("Fleet ID"), widget=forms.HiddenInput)
    name = forms.CharField(label=_("Fleet Name"))
    description = forms.CharField(
        label=_("Description"),
        widget=forms.Textarea(
            attrs={'class': 'switchable', 'data-slug': 'slug-description'})
    )

    def __init__(self, *args, **kwargs):

        super(UpdateFleetForm, self).__init__(*args, **kwargs)

        # Admin
        if policy.check((("iot", "iot:update_fleets"),), self.request):
            # LOG.debug("ADMIN")
            pass

        # Manager or Admin of the iot project
        elif (policy.check((("iot", "iot_manager"),), self.request) or
              policy.check((("iot", "iot_admin"),), self.request)):
            # LOG.debug("NO-edit IOT ADMIN")
            pass

        # Other users
        else:
            if self.request.user.id != kwargs["initial"]["owner"]:
                # LOG.debug("IMMUTABLE FIELDS")
                self.fields["name"].widget.attrs = {'readonly': 'readonly'}
                self.fields["description"].widget.attrs = {'readonly': 'readonly'}

    def handle(self, request, data):
        try:
            iotronic.fleet_update(request, data["uuid"],
                                    {"name": data["name"],
                                     "description": data["description"]})

            messages.success(request, _("Fleet updated successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to update fleet.'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

LOG = logging.getLogger(__name__)


class CreateServiceForm(forms.SelfHandlingForm):
    name = forms.CharField(label=_("Service Name"))
    port = forms.IntegerField(
        label=_("Port"),
        help_text=_("Service port")
    )

    protocol = forms.ChoiceField(
        label=_("Protocol"),
        choices=[('TCP', _('TCP')), ('UDP', _('UDP'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-protocol'},
        )
    )

    def handle(self, request, data):
        try:
            iotronic.service_create(request, data["name"],
                                    data["port"], data["protocol"])

            messages.success(request, _("Service " + str(data["name"]) +
                                        " created successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to create service.'))


class UpdateServiceForm(forms.SelfHandlingForm):
    uuid = forms.CharField(label=_("Service ID"), widget=forms.HiddenInput)
    name = forms.CharField(label=_("Service Name"))
    port = forms.IntegerField(label=_("Port"))
    protocol = forms.ChoiceField(
        label=_("Protocol"),
        choices=[('TCP', _('TCP')), ('UDP', _('UDP'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-protocol'},
        )
    )

    def __init__(self, *args, **kwargs):

        super(UpdateServiceForm, self).__init__(*args, **kwargs)

        # Admin
        if policy.check((("iot", "iot:update_services"),), self.request):
            # LOG.debug("ADMIN")
            pass

        # Manager or Admin of the iot project
        elif (policy.check((("iot", "iot_manager"),), self.request) or
              policy.check((("iot", "iot_admin"),), self.request)):
            # LOG.debug("NO-edit IOT ADMIN")
            pass

        # Other users
        else:
            if self.request.user.id != kwargs["initial"]["owner"]:
                # LOG.debug("IMMUTABLE FIELDS")
                self.fields["name"].widget.attrs = {'readonly': 'readonly'}
                self.fields["port"].widget.attrs = {'readonly': 'readonly'}
                self.fields["protocol"].widget.attrs = {'readonly': 'readonly'}

    def handle(self, request, data):
        try:
            iotronic.service_update(request, data["uuid"],
                                    {"name": data["name"],
                                     "port": data["port"],
                                     "protocol": data["protocol"]})

            messages.success(request, _("Service updated successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to update service.'))


class ServiceActionForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Service Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    board_list = forms.MultipleChoiceField(
        label=_("Boards List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-select-boards'}),
        help_text=_("Select boards in this pool")
    )

    action = forms.ChoiceField(
        label=_("Action"),
        choices=[('ServiceEnable', _('Enable')),
                 ('ServiceDisable', _('Disable')),
                 ('ServiceRestore', _('Restore'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-action'},
        )
    )

    def __init__(self, *args, **kwargs):

        super(ServiceActionForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})

        self.fields["board_list"].choices = kwargs["initial"]["board_list"]

    def handle(self, request, data):

        counter = 0

        for board in data["board_list"]:
            for key, value in self.fields["board_list"].choices:
                if key == board:

                    try:
                        action = iotronic.service_action(request, key,
                                                         data["uuid"],
                                                         data["action"])
                        message_text = action
                        messages.success(request, _(message_text))

                        if counter != len(data["board_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to execute action on board " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
# from horizon import messages
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

from iotronic_ui.iot.fleets import forms as project_forms
from iotronic_ui.iot.fleets import tables as project_tables
from iotronic_ui.iot.fleets import tabs as project_tabs


LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = project_tables.FleetsTable
    template_name = 'iot/fleets/index.html'
    page_title = _("Fleets")

    def get_data(self):
        fleets = []

        # Admin
        if policy.check((("iot", "iot:list_all_fleets"),), self.request):
            try:
                fleets = iotronic.fleet_list(self.request, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve fleets list.'))

        # Admin_iot_project
        elif policy.check((("iot", "iot:list_project_fleets"),),
                          self.request):
            try:
                fleets = iotronic.fleet_list(self.request, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user fleets list.'))

        # Other users
        else:
            try:
                fleets = iotronic.fleet_list(self.request, None)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user fleets list.'))

        return fleets


class CreateView(forms.ModalFormView):
    template_name = 'iot/fleets/create.html'
    modal_header = _("Create Fleet")
    form_id = "create_fleet_form"
    form_class = project_forms.CreateFleetForm
    submit_label = _("Create Fleet")
    submit_url = reverse_lazy("horizon:iot:fleets:create")
    success_url = reverse_lazy('horizon:iot:fleets:index')
    page_title = _("Create Fleet")


class UpdateView(forms.ModalFormView):
    template_name = 'iot/fleets/update.html'
    modal_header = _("Update Fleet")
    form_id = "update_fleet_form"
    form_class = project_forms.UpdateFleetForm
    submit_label = _("Update Fleet")
    submit_url = "horizon:iot:fleets:update"
    success_url = reverse_lazy('horizon:iot:fleets:index')
    page_title = _("Update Fleet")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.fleet_get(self.request,
                                        self.kwargs['fleet_id'],
                                        None)
        except Exception:
            redirect = reverse("horizon:iot:fleets:index")
            exceptions.handle(self.request,
                              _('Unable to get fleet information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        fleet = self.get_object()

        return {'uuid': fleet.uuid,
                'name': fleet.name,
                'description': fleet.description}


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.FleetDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ fleet.name|default:fleet.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        fleet = self.get_data()
        context["fleet"] = fleet
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(fleet)

        return context

    def _get_actions(self, fleet):
        table = project_tables.FleetsTable(self.request)
        return table.render_row_actions(fleet)

    @memoized.memoized_method
    def get_data(self):
        fleet = []
        fleet_boards = []

        fleet_id = self.kwargs['fleet_id']
        try:
            fleet = iotronic.fleet_get(self.request, fleet_id, None)
            boards = iotronic.fleet_get_boards(self.request, fleet_id)

            # LOG.debug('Boards: %s', boards)

            for board in boards:
                fleet_boards.append(board._info)

            fleet._info.update(dict(boards=fleet_boards))
            # LOG.debug('FLEET COMPLETE: %s', fleet)

        except Exception:
            s = fleet.name
            msg = ('Unable to retrieve fleet %s information') % {'name': s}
            exceptions.handle(self.request, msg, ignore=True)
        return fleet

    def get_tabs(self, request, *args, **kwargs):
        fleet = self.get_data()
        return self.tab_group_class(request, fleet=fleet, **kwargs)


class FleetDetailView(DetailView):
    redirect_url = 'horizon:iot:fleets:index'

    def _get_actions(self, fleet):
        table = project_tables.FleetsTable(self.request)
        return table.render_row_actions(fleet)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

# from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tabs

LOG = logging.getLogger(__name__)


"""
class OverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = ("iot/webservices/_detail_overview.html")

    def get_context_data(self, request):
        # coordinates = self.tab_group.kwargs['board'].__dict__["location"][0]
        # LOG.debug('IOT INFO: %s', coordinates)

        board_name = self.tab_group.kwargs['webservice']._info['board_name']
        board_uuid = self.tab_group.kwargs['webservice']._info['board_uuid']

        return {"webservice": self.tab_group.kwargs['webservice'],
                "board_uuid": board_uuid,
                "board_name": board_name,
                "is_superuser": request.user.is_superuser}


class WebserviceDetailTabs(tabs.TabGroup):
    slug = "webservice_details"
    # tabs = (OverviewTab, LogTab, ConsoleTab, AuditTab)
    tabs = (OverviewTab,)
    sticky = True
"""



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from openstack_dashboard import api

LOG = logging.getLogger(__name__)


class CreateServiceLink(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Service")
    url = "horizon:iot:services:create"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:create_service"),)


class EditServiceLink(tables.LinkAction):
    name = "edit"
    verbose_name = _("Edit")
    url = "horizon:iot:services:update"
    classes = ("ajax-modal",)
    icon = "pencil"
    # policy_rules = (("iot", "iot:update_service"),)


class ActionServiceLink(tables.LinkAction):
    name = "action"
    verbose_name = _("Service Action")
    url = "horizon:iot:services:action"
    classes = ("ajax-modal",)
    # icon = "plus"
    # policy_rules = (("iot", "iot:service_action"),)


class DeleteServicesAction(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Service",
            u"Delete Services",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Service",
            u"Deleted Services",
            count
        )
    # policy_rules = (("iot", "iot:delete_service"),)

    def delete(self, request, service_id):
        api.iotronic.service_delete(request, service_id)


class ServiceFilterAction(tables.FilterAction):

    def filter(self, table, services, filter_string):
        # Naive case-insensitive search.
        q = filter_string.lower()
        return [service for service in services
                if q in service.name.lower()]


class ServicesTable(tables.DataTable):
    name = tables.WrappingColumn('name', link="horizon:iot:services:detail",
                                 verbose_name=_('Service Name'))
    protocol = tables.Column('protocol', verbose_name=_('Protocol'))
    port = tables.Column('port', verbose_name=_('Port'))

    # Overriding get_object_id method because in IoT service the "id" is
    # identified by the field UUID
    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "services"
        verbose_name = _("services")
        row_actions = (EditServiceLink, ActionServiceLink,
                       DeleteServicesAction)
        table_actions = (ServiceFilterAction, CreateServiceLink,
                         DeleteServicesAction)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

LOG = logging.getLogger(__name__)


class CreateBoardForm(forms.SelfHandlingForm):
    name = forms.CharField(label=_("Board Name"))
    code = forms.CharField(
        label=_("Registration Code"),
        help_text=_("Registration code")
    )

    # MODIFY ---> options: yun, server
    type = forms.ChoiceField(
        label=_("Type"),
        # choices=[('yun', _('YUN')), ('server', _('Server'))],
        choices=[('gateway', _('Gateway')), ('server', _('Server'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-type'},
        )
    )

    """
    mobile = forms.ChoiceField(
        label=_("Mobile"),
        choices =[('false', _('False')), ('true', _('True'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-mobile'},
        )
    )
    """
    mobile = forms.BooleanField(label=_("Mobile"), required=False)

    latitude = forms.FloatField(label=_("Latitude"))
    longitude = forms.FloatField(label=_("Longitude"))
    altitude = forms.FloatField(label=_("Altitude"))

    def handle(self, request, data):
        try:

            # Float
            # data["location"] = [{"latitude": data["latitude"],
            #                      "longitude": data["longitude"],
            #                      "altitude": data["altitude"]}]
            # String
            data["location"] = [{"latitude": str(data["latitude"]),
                                 "longitude": str(data["longitude"]),
                                 "altitude": str(data["altitude"])}]

            iotronic.board_create(request, data["code"],
                                  data["mobile"], data["location"],
                                  data["type"], data["name"])

            messages.success(request, _("Board created successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to create board.'))


class UpdateBoardForm(forms.SelfHandlingForm):
    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)
    name = forms.CharField(label=_("Board Name"))
    
    fleet_list = forms.ChoiceField(
        label=_("Fleets List"),
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-fleet'}),
        help_text=_("Select fleet in this pool "),
        required=False
    )

    mobile = forms.BooleanField(label=_("Mobile"), required=False)

    """
    latitude = forms.FloatField(label=_("Latitude"))
    longitude = forms.FloatField(label=_("Longitude"))
    altitude = forms.FloatField(label=_("Altitude"))
    """

    def __init__(self, *args, **kwargs):

        super(UpdateBoardForm, self).__init__(*args, **kwargs)

        # Populate fleets
        fleets = iotronic.fleet_list(self.request, None)
        fleets.sort(key=lambda b: b.name)

        fleet_list = []
        fleet_list.append((None, _("-")))
        for fleet in fleets:
            fleet_list.append((fleet.uuid, _(fleet.name)))

        # LOG.debug("FLEETS: %s", fleet_list)
        self.fields["fleet_list"].choices = fleet_list
        self.fields["fleet_list"].initial = kwargs["initial"]["fleet_id"]

        # LOG.debug("INITIAL: %s", kwargs["initial"])

        # LOG.debug("Manager: %s", policy.check((("iot", "iot_manager"),),
        #                                            self.request))
        # LOG.debug("Admin: %s", policy.check((("iot", "iot_admin"),),
        #                                          self.request))

        # Admin
        if policy.check((("iot", "iot:update_boards"),), self.request):
            # LOG.debug("ADMIN")
            pass

        # Manager or Admin of the iot project
        elif (policy.check((("iot", "iot_manager"),), self.request) or
              policy.check((("iot", "iot_admin"),), self.request)):
            # LOG.debug("NO-edit IOT ADMIN")
            pass

        # Other users
        else:
            if self.request.user.id != kwargs["initial"]["owner"]:
                # LOG.debug("IMMUTABLE FIELDS")
                self.fields["name"].widget.attrs = {'readonly': 'readonly'}
                self.fields["mobile"].widget.attrs = {'disabled': 'disabled'}
                self.fields["fleet_list"].widget.attrs = {'disabled':
                                                          'disabled'}

                """
                self.fields["latitude"].widget.attrs = {'readonly':
                                                        'readonly'}
                self.fields["longitude"].widget.attrs = {'readonly':
                                                         'readonly'}
                self.fields["altitude"].widget.attrs = {'readonly':
                                                        'readonly'}
                """

    def handle(self, request, data):

        try:

            # data["location"] = [{"latitude": str(data["latitude"]),
            #                      "longitude": str(data["longitude"]),
            #                      "altitude": str(data["altitude"])}]
            # iotronic.board_update(request, data["uuid"],
            #                       {"name": data["name"],
            #                        "mobile": data["mobile"],
            #                        "location": data["location"]})
            if data["fleet_list"] == '':
                data["fleet_list"] = None

            iotronic.board_update(request, data["uuid"],
                                  {"name": data["name"],
                                   "fleet": data["fleet_list"],
                                   "mobile": data["mobile"]})
            messages.success(request, _("Board updated successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to update board.'))


class EnableServiceForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    service_list = forms.MultipleChoiceField(
        label=_("Services List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-select-services'}),
        help_text=_("Add available services from this pool")
    )

    def __init__(self, *args, **kwargs):
        super(EnableServiceForm, self).__init__(*args, **kwargs)
        self.fields["service_list"].choices = kwargs["initial"]["service_list"]

    def handle(self, request, data):

        counter = 0
        for service in data["service_list"]:
            try:
                action = iotronic.service_action(request, data["uuid"],
                                                 service, "ServiceEnable")

                # message_text = "Service(s) enabled successfully."
                message_text = action
                messages.success(request, _(message_text))

                if counter != len(data["service_list"]) - 1:
                    counter += 1
                else:
                    return True

            except Exception:
                message_text = "Unable to enable service."
                exceptions.handle(request, _(message_text))


class DisableServiceForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    service_list = forms.MultipleChoiceField(
        label=_("Services List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-select-services'}),
        help_text=_("Select services to disable from this pool")
    )

    def __init__(self, *args, **kwargs):
        super(DisableServiceForm, self).__init__(*args, **kwargs)
        self.fields["service_list"].choices = kwargs["initial"]["service_list"]

    def handle(self, request, data):

        counter = 0
        for service in data["service_list"]:
            try:
                action = iotronic.service_action(request, data["uuid"],
                                                 service, "ServiceDisable")

                # message_text = "Service(s) disabled successfully."
                message_text = action
                messages.success(request, _(message_text))

                if counter != len(data["service_list"]) - 1:
                    counter += 1
                else:
                    return True

            except Exception:
                message_text = "Unable to disable service."
                exceptions.handle(request, _(message_text))


class AttachPortForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    networks_list = forms.ChoiceField(
        label=_("Networks List"),
        help_text=_("Select network:subnet from the list")
    )

    def __init__(self, *args, **kwargs):

        super(AttachPortForm, self).__init__(*args, **kwargs)

        net_choices = kwargs["initial"]["networks_list"]
        self.fields["networks_list"].choices = net_choices

    def handle(self, request, data):
        array = data["networks_list"].split(':')
        LOG.debug(array)
        network_id = array[0]
        subnet_id = array[1]

        try:
            attach = iotronic.attach_port(request, data["uuid"],
                                          network_id, subnet_id)

            # LOG.debug("ATTACH: %s", attach)
            ip = attach._info["ip"]

            message_text = "Attached  port to ip " + str(ip) + \
                           " on board " + str(data["name"]) + \
                           " completed successfully"
            messages.success(request, _(message_text))
            return True

        except Exception:
            message_text = "Unable to attach port on board " + \
                           str(data["name"])

            exceptions.handle(request, _(message_text))


class DetachPortForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    port_list = forms.MultipleChoiceField(
        label=_("Ports List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-detacj-ports'}),
        help_text=_("Select one or more of the following attached ports")
    )

    def __init__(self, *args, **kwargs):

        super(DetachPortForm, self).__init__(*args, **kwargs)
        self.fields["port_list"].choices = kwargs["initial"]["ports"]

    def handle(self, request, data):
        # LOG.debug("DATA: %s %s", data, len(data["port_list"]))

        counter = 0

        for port in data["port_list"]:
            try:
                iotronic.detach_port(request, data["uuid"], port)

                message_text = "Detach port " + str(port) + " from board " + \
                               str(data["name"]) + " completed successfully"
                messages.success(request, _(message_text))

                if counter != len(data["port_list"]) - 1:
                    counter += 1
                else:
                    return True

            except Exception:
                message_text = "Unable to detach port " + str(port) + \
                               " from board " + str(data["name"])

                exceptions.handle(request, _(message_text))


class EnableWebServiceForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    dns = forms.CharField(label=_("Domain Name Server"))
    zone = forms.CharField(label=_("Zone"))
    email = forms.CharField(label=_("Email"))

    def __init__(self, *args, **kwargs):
        super(EnableWebServiceForm, self).__init__(*args, **kwargs)

    def handle(self, request, data):

        try:
            iotronic.webservice_enable(request, data["uuid"],
                                       data["dns"], data["zone"],
                                       data["email"])

            messages.success(request, _("Web Service enabled on board " +
                                        str(data["name"]) + "."))
            return True

        except Exception:
            message_text = "Unable to enable web service."
            exceptions.handle(request, _(message_text))


class DisableWebServiceForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    def __init__(self, *args, **kwargs):
        super(DisableWebServiceForm, self).__init__(*args, **kwargs)

    def handle(self, request, data):

        try:
            iotronic.webservice_disable(request, data["uuid"])

            messages.success(request, _("Web Service disabled on board " +
                                        str(data["name"]) + "."))
            return True

        except Exception:
            message_text = "Unable to disable web service."
            exceptions.handle(request, _(message_text))


class RemovePluginsForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    plugin_list = forms.MultipleChoiceField(
        label=_("Plugins List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-remove-plugins'}),
        help_text=_("Select plugins in this pool ")
    )

    def __init__(self, *args, **kwargs):

        super(RemovePluginsForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})
        self.fields["plugin_list"].choices = kwargs["initial"]["plugin_list"]

    def handle(self, request, data):

        counter = 0

        for plugin in data["plugin_list"]:
            for key, value in self.fields["plugin_list"].choices:
                if key == plugin:

                    try:
                        iotronic.plugin_remove(request, data["uuid"], key)

                        message_text = "Plugin " + str(value) + \
                                       " removed successfully."
                        messages.success(request, _(message_text))

                        if counter != len(data["plugin_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to remove plugin " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

# from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tabs

LOG = logging.getLogger(__name__)

"""
import inspect
LOG.debug('CLASSES: %s',
          inspect.getmembers(tabs, predicate=inspect.isclass))
"""


class OverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = ("iot/boards/_detail_overview.html")

    def get_context_data(self, request):

        coordinates = self.tab_group.kwargs['board'].__dict__['location'][0]
        ports = self.tab_group.kwargs['board']._info['ports']
        services = self.tab_group.kwargs['board']._info['services']
        webservices = self.tab_group.kwargs['board']._info['webservices']
        plugins = self.tab_group.kwargs['board']._info['plugins']

        return {"board": self.tab_group.kwargs['board'],
                "coordinates": coordinates,
                "services": services,
                "webservices": webservices,
                "ports": ports,
                "plugins": plugins,
                "is_superuser": request.user.is_superuser}


class BoardDetailTabs(tabs.TabGroup):
    slug = "board_details"
    # tabs = (OverviewTab, LogTab, ConsoleTab, AuditTab)
    tabs = (OverviewTab,)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

LOG = logging.getLogger(__name__)


class ExposeWebserviceForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    zone = forms.CharField(label=_("Zone"),
                           widget=forms.TextInput(attrs={'readonly':
                                                         'readonly'}))

    ws_name = forms.CharField(label=_("Web Service Name"))

    port = forms.IntegerField(
        label=_("Port"),
        help_text=_("The local port used by the service")
    )

    secure = forms.BooleanField(label=_("Secure"), initial=True,
                                widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        super(ExposeWebserviceForm, self).__init__(*args, **kwargs)

    def handle(self, request, data):
        try:
            iotronic.webservice_expose(request, data["uuid"],
                                       data["ws_name"], data["port"],
                                       data["secure"])

            messages.success(request, _("Web Service " + str(data["name"]) +
                                        " exposed successfully on port " +
                                        str(data["port"]) + "."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to expose web service.'))


class UnexposeWebserviceForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Board ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Board Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    ws_onboard = forms.MultipleChoiceField(
        label=_("Web Services on board"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-select-webservices'}),
        help_text=_("Select a webservice from the list")
    )

    def __init__(self, *args, **kwargs):

        super(UnexposeWebserviceForm, self).__init__(*args, **kwargs)
        self.fields["ws_onboard"].choices = kwargs["initial"]["ws_onboard"]

    def handle(self, request, data):

        counter = 0
        for ws in data["ws_onboard"]:
            try:
                iotronic.webservice_unexpose(request, ws)

                message_text = "Web Service(s) unexposed successfully."
                messages.success(request, _(message_text))

                if counter != len(data["ws_onboard"]) - 1:
                    counter += 1
                else:
                    return True

            except Exception:
                message_text = "Unable to unexpose web service."
                exceptions.handle(request, _(message_text))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# from collections import OrderedDict
# import threading

from iotronicclient import client as iotronic_client
# from django.conf import settings
from django.utils.translation import ugettext_lazy as _

# from horizon import exceptions
from horizon.utils.memoized import memoized  # noqa

from openstack_dashboard.api import base
# from openstack_dashboard.api import keystone


# TESTING
import logging
LOG = logging.getLogger(__name__)


@memoized
def iotronicclient(request):
    """Initialization of Iotronic client."""

    endpoint = base.url_for(request, 'iot')
    # insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    # cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)

    return iotronic_client.Client('1', endpoint, token=request.user.token.id)


# BOARD MANAGEMENT
def board_list(request, status=None, detail=None, project=None):
    """List boards."""
    return iotronicclient(request).board.list(status, detail, project)


def board_get(request, board_id, fields):
    """Get board info."""
    return iotronicclient(request).board.get(board_id, fields)


def board_create(request, code, mobile, location, type, name):
    """Create board."""
    params = {"code": code,
              "mobile": mobile,
              "location": location,
              "type": type,
              "name": name}
    iotronicclient(request).board.create(**params)


def board_update(request, board_id, patch):
    """Update board."""
    iotronicclient(request).board.update(board_id, patch)


def board_delete(request, board_id):
    """Delete board."""
    iotronicclient(request).board.delete(board_id)


# PLUGIN MANAGEMENT (Cloud Side)
def plugin_list(request, detail=None, project=None, with_public=False,
                all_plugins=False):
    """List plugins."""
    return iotronicclient(request).plugin.list(detail, project,
                                               with_public=with_public,
                                               all_plugins=all_plugins)


def plugin_get(request, plugin_id, fields):
    """Get plugin info."""
    plugin = iotronicclient(request).plugin.get(plugin_id, fields)
    return plugin


def plugin_create(request, name, public, callable, code, parameters):
    """Create plugin."""
    params = {"name": name,
              "public": public,
              "callable": callable,
              "code": code,
              "parameters": parameters}
    iotronicclient(request).plugin.create(**params)


def plugin_update(request, plugin_id, patch):
    """Update plugin."""
    iotronicclient(request).plugin.update(plugin_id, patch)


def plugin_delete(request, plugin_id):
    """Delete plugin."""
    return iotronicclient(request).plugin.delete(plugin_id)


# PLUGIN MANAGEMENT (Board Side)
def plugin_inject(request, board_id, plugin_id, onboot):
    """Inject plugin on board(s)."""
    return iotronicclient(request).plugin_injection.plugin_inject(board_id,
                                                                  plugin_id,
                                                                  onboot)


def plugin_action(request, board_id, plugin_id, action, params={}):
    """Start/Stop/Call actions on board(s)."""
    return iotronicclient(request).plugin_injection.plugin_action(
        board_id, plugin_id, action, params)


def plugin_remove(request, board_id, plugin_id):
    """Remove plugin from board."""
    iotronicclient(request).plugin_injection.plugin_remove(board_id,
                                                           plugin_id)


def plugins_on_board(request, board_id):
    """Plugins on board."""
    plugins = iotronicclient(request).plugin_injection.plugins_on_board(
        board_id)

    detailed_plugins = []
    # fields = {"name", "public", "callable"}
    fields = {"name"}
    for plugin in plugins:
        details = iotronicclient(request).plugin.get(plugin.plugin, fields)
        detailed_plugins.append({"name": details._info["name"],
                                 "id": plugin.plugin})

    return detailed_plugins


# SERVICE MANAGEMENT
def service_list(request, detail=None):
    """List services."""
    return iotronicclient(request).service.list(detail)


def service_get(request, service_id, fields):
    """Get service info."""
    return iotronicclient(request).service.get(service_id, fields)


def service_create(request, name, port, protocol):
    """Create service."""
    params = {"name": name,
              "port": port,
              "protocol": protocol}
    iotronicclient(request).service.create(**params)


def service_update(request, service_id, patch):
    """Update service."""
    iotronicclient(request).service.update(service_id, patch)


def service_delete(request, service_id):
    """Delete service."""
    iotronicclient(request).service.delete(service_id)


def services_on_board(request, board_id, detail=False):
    """List services on board."""
    services = iotronicclient(request).exposed_service.services_on_board(
        board_id)

    if detail:
        detailed_services = []
        fields = {"name", "port", "protocol"}

        for service in services:
            details = iotronicclient(request).service.get(
                service._info["service"], fields)

            detailed_services.append({"uuid": service._info["service"],
                                      "name": details._info["name"],
                                      "public_port":
                                          service._info["public_port"],
                                      "port": details._info["port"],
                                      "protocol": details._info["protocol"]})

        return detailed_services

    else:
        return services


def service_action(request, board_id, service_id, action):
    """Action on service."""
    return iotronicclient(request).exposed_service.service_action(board_id,
                                                                  service_id,
                                                                  action)


def restore_services(request, board_id):
    """Restore services."""
    return iotronicclient(request).exposed_service.restore_services(board_id)


# PORTS MANAGEMENT
def port_list(request, board_id):
    """Get ports attached to a board."""
    return iotronicclient(request).port.list()


def attach_port(request, board_id, network_id, subnet_id):
    """Attach port to a subnet for a board."""
    return iotronicclient(request).portonboard.attach_port(board_id,
                                                           network_id,
                                                           subnet_id)


def detach_port(request, board_id, port_id):
    """Detach port from the board."""
    iotronicclient(request).portonboard.detach_port(board_id, port_id)


# FLEETS MANAGEMENT
def fleet_list(request, detail=None):
    """Get fleets list."""
    return iotronicclient(request).fleet.list()


def fleet_get(request, fleet_id, fields):
    """Get fleet info."""
    return iotronicclient(request).fleet.get(fleet_id, fields)


def fleet_create(request, name, description):
    """Create fleet."""
    params = {"name": name,
              "description": description}

    iotronicclient(request).fleet.create(**params)


def fleet_delete(request, fleet_id):
    """Delete fleet."""
    iotronicclient(request).fleet.delete(fleet_id)


def fleet_update(request, fleet_id, patch):
    """Update fleet."""
    iotronicclient(request).fleet.update(fleet_id, patch)


def fleet_get_boards(request, fleet_id):
    """Get fleet boards."""
    return iotronicclient(request).fleet.boards_in_fleet(fleet=fleet_id)


# WEBSERVICES MANAGEMENT
def webservice_list(request, detail=None):
    """Get web services list."""
    return iotronicclient(request).webservice.list()


def webservice_enabled_list(request):
    """Get enabled web services list."""
    return iotronicclient(request).enabledwebservice.list()


def webservice_get_enabled_info(request, board_id, detail=None):
    """Get the information of the enabled webservices."""
    ws_info = []

    ws_enabled = iotronicclient(request).enabledwebservice.list()

    for ws in ws_enabled:
        if ws.board_uuid == board_id:
            ws_info = ws
            break

    return ws_info


def webservices_on_board(request, board_id, fields=None):
    """Get web services on board list."""
    webservices = iotronicclient(request).webserviceonboard.list(board_id,
                                                                 fields)

    detailed_webservices = []
    # fields = {"name", "port", "uuid"}

    for ws in webservices:
        detailed_webservices.append({"name": ws._info["name"],
                                     "port": ws._info["port"],
                                     "uuid": ws._info["uuid"]})

    return detailed_webservices


def webservice_get(request, webservice_id, fields):
    """Get web service info."""
    return iotronicclient(request).webservice.get(webservice_id, fields)


def webservice_expose(request, board_id, name, port, secure):
    """Expose a web service."""
    return iotronicclient(request).webserviceonboard.expose(board_id,
                                                            name,
                                                            port,
                                                            secure)


def webservice_unexpose(request, webservice_id):
    """Unexpose a web service from a board."""
    return iotronicclient(request).webservice.delete(webservice_id)


def webservice_enable(request, board, dns, zone, email):
    """Enable web service."""
    return iotronicclient(request).webserviceonboard.enable_webservice(board,
                                                                       dns,
                                                                       zone,
                                                                       email)


def webservice_disable(request, board):
    """Disable web service."""
    return iotronicclient(request).webserviceonboard.disable_webservice(board)


def boards_no_webservice(request):
    """Get all the boards that have not webservice enabled."""

    boards_no_ws_enabled = []

    board_list = iotronicclient(request).board.list()
    board_list.sort(key=lambda b: b.name)

    board_ws_list = iotronicclient(request).enabledwebservice.list()

    for board in board_list:
        for i in range(len(board_ws_list)):
            if board.uuid == board_ws_list[i].board_uuid:
                break
            elif ((board.uuid != board_ws_list[i].board_uuid) and
                 (i==len(board_ws_list)-1)):
                boards_no_ws_enabled.append((board.uuid, _(board.name)))

    # LOG.debug('COMPLEMENTARY %s', boards_no_ws_enabled)
    return boards_no_ws_enabled



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from openstack_dashboard import api

LOG = logging.getLogger(__name__)


class CreateFleetLink(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Fleet")
    url = "horizon:iot:fleets:create"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:create_fleet"),)


class EditFleetLink(tables.LinkAction):
    name = "edit"
    verbose_name = _("Edit")
    url = "horizon:iot:fleets:update"
    classes = ("ajax-modal",)
    icon = "pencil"
    # policy_rules = (("iot", "iot:update_fleet"),)


class DeleteFleetsAction(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Fleet",
            u"Delete Fleets",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Fleet",
            u"Deleted Fleets",
            count
        )
    # policy_rules = (("iot", "iot:delete_fleet"),)

    def delete(self, request, fleet_id):
        api.iotronic.fleet_delete(request, fleet_id)


class FleetFilterAction(tables.FilterAction):

    def filter(self, table, fleets, filter_string):
        # Naive case-insensitive search.
        q = filter_string.lower()
        return [fleet for fleet in fleets
                if q in fleet.name.lower()]


class FleetsTable(tables.DataTable):
    name = tables.WrappingColumn('name', link="horizon:iot:fleets:detail",
                                 verbose_name=_('Fleet Name'))
    description = tables.Column('description', verbose_name=_('Description'))

    # Overriding get_object_id method because in IoT fleet the "id" is
    # identified by the field UUID
    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "fleets"
        verbose_name = _("fleets")
        row_actions = (EditFleetLink, DeleteFleetsAction)
        table_actions = (FleetFilterAction, CreateFleetLink,
                         DeleteFleetsAction)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# The slug of the panel to be added to HORIZON_CONFIG. Required.
PANEL = 'services'
# The slug of the dashboard the PANEL associated with. Required.
PANEL_DASHBOARD = 'iot'
# The slug of the panel group the PANEL is associated with.
PANEL_GROUP = 'iot'
# If set, it will update the default panel of the PANEL_DASHBOARD.
DEFAULT_PANEL = ''

# Python panel class of the PANEL to be added.
ADD_PANEL = 'iotronic_ui.iot.services.panel.Services'



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from openstack_dashboard import api

LOG = logging.getLogger(__name__)


class CreatePluginLink(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Plugin")
    url = "horizon:iot:plugins:create"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:create_plugin"),)


class EditPluginLink(tables.LinkAction):
    name = "edit"
    verbose_name = _("Edit")
    url = "horizon:iot:plugins:update"
    classes = ("ajax-modal",)
    icon = "pencil"
    # policy_rules = (("iot", "iot:update_plugin"),)

    """
    def allowed(self, request, plugin):
        # LOG.debug("ALLOWED: %s %s %s", self, request, plugin)
        # LOG.debug("user: %s", request.user.id)

        return True
    """


class InjectPluginLink(tables.LinkAction):
    name = "inject"
    verbose_name = _("Inject Plugin")
    url = "horizon:iot:plugins:inject"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:inject_plugin"),)


class StartPluginLink(tables.LinkAction):
    name = "start"
    verbose_name = _("Start Plugin")
    url = "horizon:iot:plugins:start"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:start_plugin"),)


class StopPluginLink(tables.LinkAction):
    name = "stop"
    verbose_name = _("Stop Plugin")
    url = "horizon:iot:plugins:stop"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:stop_plugin"),)


class CallPluginLink(tables.LinkAction):
    name = "call"
    verbose_name = _("Call Plugin")
    url = "horizon:iot:plugins:call"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:call_plugin"),)


class RemovePluginLink(tables.LinkAction):
    name = "remove"
    verbose_name = _("Remove Plugin from board(s)")
    url = "horizon:iot:plugins:remove"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:remove_plugin"),)


class DeletePluginsAction(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Plugin",
            u"Delete Plugins",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Plugin",
            u"Deleted Plugins",
            count
        )
    # policy_rules = (("iot", "iot:delete_plugin"),)

    """
    def allowed(self, request, role):
        return api.keystone.keystone_can_edit_role()
    """

    def delete(self, request, plugin_id):
        api.iotronic.plugin_delete(request, plugin_id)


class PluginFilterAction(tables.FilterAction):

    def filter(self, table, plugins, filter_string):
        """Naive case-insensitive search."""
        q = filter_string.lower()
        return [plugin for plugin in plugins
                if q in plugin.name.lower()]


class PluginsTable(tables.DataTable):
    name = tables.WrappingColumn('name', link="horizon:iot:plugins:detail",
                                 verbose_name=_('Plugin Name'))
    uuid = tables.Column('uuid', verbose_name=_('Plugin ID'))
    owner = tables.Column('owner', verbose_name=_('Owner'))
    public = tables.Column('public', verbose_name=_('Public'))
    callable = tables.Column('callable', verbose_name=_('Callable'))

    # Overriding get_object_id method because in IoT service the "id" is
    # identified by the field UUID
    def get_object_id(self, datum):
        # LOG.debug("datum %s", datum)
        return datum.uuid

    # Overriding get_row_actions method because we need to discriminate
    # between Sync and Async plugins
    def get_row_actions(self, datum):
        actions = super(PluginsTable, self).get_row_actions(datum)
        # LOG.debug("ACTIONS: %s %s", actions[0].name, datum.name)

        selected_row_actions = []

        common_actions = ["edit", "inject", "remove", "delete"]

        for action in actions:
            if action.name in common_actions:
                selected_row_actions.append(action)

            elif datum.callable == True and action.name == "call":
                selected_row_actions.append(action)

            elif datum.callable == False and action.name != "call":
                selected_row_actions.append(action)

        return selected_row_actions

    class Meta(object):
        name = "plugins"
        verbose_name = _("plugins")

        row_actions = (EditPluginLink, InjectPluginLink, StartPluginLink,
                       StopPluginLink, CallPluginLink, RemovePluginLink,
                       DeletePluginsAction,)
        table_actions = (PluginFilterAction, CreatePluginLink,
                         DeletePluginsAction)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import cPickle
import logging

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
# from horizon import messages
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from openstack_dashboard.api import iotronic, keystone
from openstack_dashboard import policy

from iotronic_ui.iot.plugins import forms as project_forms
from iotronic_ui.iot.plugins import tables as project_tables
from iotronic_ui.iot.plugins import tabs as project_tabs

LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = project_tables.PluginsTable
    template_name = 'iot/plugins/index.html'
    page_title = _("Plugins")

    def get_data(self):
        plugins = []
        users = []

        # Admin
        if policy.check((("iot", "iot:list_all_plugins"),), self.request):
            try:
                plugins = iotronic.plugin_list(self.request, None, None,
                                               all_plugins=True)
                users = keystone.user_list(self.request)

            except Exception:
                exceptions.handle(self.request, _('Unable to retrieve plugins \
                                  list.'))

        # Admin_iot_project
        elif policy.check((("iot", "iot:list_project_plugins"),),
                          self.request):
            try:
                plugins = iotronic.plugin_list(self.request, None, None,
                                               with_public=True)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user plugins list.'))

        # Other users
        else:
            try:
                plugins = iotronic.plugin_list(self.request, None, None,
                                               with_public=True)

            except Exception:
                exceptions.handle(self.request,
                                  _('Unable to retrieve user plugins list.'))

        # Replace owner column values (user.id) with user.name (only Admin
        # can see human-readable names)
        for plugin in plugins:
            for user in users:
                if plugin.owner == user.id:
                    plugin.owner = user.name
                    break

        return plugins


class CreateView(forms.ModalFormView):
    template_name = 'iot/plugins/create.html'
    modal_header = _("Create Plugin")
    form_id = "create_plugin_form"
    form_class = project_forms.CreatePluginForm
    submit_label = _("Create Plugin")
    submit_url = reverse_lazy("horizon:iot:plugins:create")
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Create Plugin")


class InjectView(forms.ModalFormView):
    template_name = 'iot/plugins/inject.html'
    modal_header = _("Inject Plugin")
    form_id = "inject_plugin_form"
    form_class = project_forms.InjectPluginForm
    submit_label = _("Inject Plugin")
    # submit_url = reverse_lazy("horizon:iot:plugins:inject")
    submit_url = "horizon:iot:plugins:inject"
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Inject Plugin")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.plugin_get(self.request, self.kwargs['plugin_id'],
                                       None)
        except Exception:
            redirect = reverse("horizon:iot:plugins:index")
            exceptions.handle(self.request,
                              _('Unable to get plugin information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(InjectView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        plugin = self.get_object()

        # Populate boards
        boards = iotronic.board_list(self.request, "online", None, None)
        boards.sort(key=lambda b: b.name)

        board_list = []
        for board in boards:
            board_list.append((board.uuid, _(board.name)))

        return {'uuid': plugin.uuid,
                'name': plugin.name,
                'board_list': board_list}


class StartView(forms.ModalFormView):
    template_name = 'iot/plugins/start.html'
    modal_header = _("Start Plugin")
    form_id = "start_plugin_form"
    form_class = project_forms.StartPluginForm
    submit_label = _("Start Plugin")
    # submit_url = reverse_lazy("horizon:iot:plugins:start")
    submit_url = "horizon:iot:plugins:start"
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Start Plugin")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.plugin_get(self.request, self.kwargs['plugin_id'],
                                       None)
        except Exception:
            redirect = reverse("horizon:iot:plugins:index")
            exceptions.handle(self.request,
                              _('Unable to get plugin information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(StartView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        plugin = self.get_object()

        # Populate boards
        boards = iotronic.board_list(self.request, "online", None, None)
        boards.sort(key=lambda b: b.name)

        board_list = []
        for board in boards:
            board_list.append((board.uuid, _(board.name)))

        return {'uuid': plugin.uuid,
                'name': plugin.name,
                'board_list': board_list}


class StopView(forms.ModalFormView):
    template_name = 'iot/plugins/stop.html'
    modal_header = _("Stop Plugin")
    form_id = "stop_plugin_form"
    form_class = project_forms.StopPluginForm
    submit_label = _("Stop Plugin")
    # submit_url = reverse_lazy("horizon:iot:plugins:stop")
    submit_url = "horizon:iot:plugins:stop"
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Stop Plugin")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.plugin_get(self.request, self.kwargs['plugin_id'],
                                       None)
        except Exception:
            redirect = reverse("horizon:iot:plugins:index")
            exceptions.handle(self.request,
                              _('Unable to get plugin information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(StopView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        plugin = self.get_object()

        # Populate boards
        boards = iotronic.board_list(self.request, "online", None, None)
        boards.sort(key=lambda b: b.name)

        board_list = []
        for board in boards:
            board_list.append((board.uuid, _(board.name)))

        return {'uuid': plugin.uuid,
                'name': plugin.name,
                'board_list': board_list}


class CallView(forms.ModalFormView):
    template_name = 'iot/plugins/call.html'
    modal_header = _("Call Plugin")
    form_id = "call_plugin_form"
    form_class = project_forms.CallPluginForm
    submit_label = _("Call Plugin")
    # submit_url = reverse_lazy("horizon:iot:plugins:call")
    submit_url = "horizon:iot:plugins:call"
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Call Plugin")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.plugin_get(self.request, self.kwargs['plugin_id'],
                                       None)
        except Exception:
            redirect = reverse("horizon:iot:plugins:index")
            exceptions.handle(self.request,
                              _('Unable to get plugin information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(CallView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        plugin = self.get_object()

        # Populate boards
        boards = iotronic.board_list(self.request, "online", None, None)
        boards.sort(key=lambda b: b.name)

        board_list = []
        for board in boards:
            board_list.append((board.uuid, _(board.name)))

        return {'uuid': plugin.uuid,
                'name': plugin.name,
                'board_list': board_list}


class RemoveView(forms.ModalFormView):
    template_name = 'iot/plugins/remove.html'
    modal_header = _("Remove Plugin")
    form_id = "remove_plugin_form"
    form_class = project_forms.RemovePluginForm
    submit_label = _("Remove Plugin")
    # submit_url = reverse_lazy("horizon:iot:plugins:remove")
    submit_url = "horizon:iot:plugins:remove"
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Remove Plugin")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.plugin_get(self.request, self.kwargs['plugin_id'],
                                       None)
        except Exception:
            redirect = reverse("horizon:iot:plugins:index")
            exceptions.handle(self.request,
                              _('Unable to get plugin information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(RemoveView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        plugin = self.get_object()

        # Populate boards
        boards = iotronic.board_list(self.request, "online", None, None)
        boards.sort(key=lambda b: b.name)

        board_list = []
        for board in boards:
            board_list.append((board.uuid, _(board.name)))

        return {'uuid': plugin.uuid,
                'name': plugin.name,
                'board_list': board_list}


class UpdateView(forms.ModalFormView):
    template_name = 'iot/plugins/update.html'
    modal_header = _("Update Plugin")
    form_id = "update_plugin_form"
    form_class = project_forms.UpdatePluginForm
    submit_label = _("Update Plugin")
    submit_url = "horizon:iot:plugins:update"
    success_url = reverse_lazy('horizon:iot:plugins:index')
    page_title = _("Update Plugin")

    @memoized.memoized_method
    def get_object(self):
        try:
            return iotronic.plugin_get(self.request, self.kwargs['plugin_id'],
                                       None)
        except Exception:
            redirect = reverse("horizon:iot:plugins:index")
            exceptions.handle(self.request,
                              _('Unable to get plugin information.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        args = (self.get_object().uuid,)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        plugin = self.get_object()

        # Deserialize the code of the plugin
        plugin.code = cPickle.loads(str(plugin.code))

        return {'uuid': plugin.uuid,
                'owner': plugin.owner,
                'name': plugin.name,
                'public': plugin.public,
                'callable': plugin.callable,
                'code': plugin.code}


class DetailView(tabs.TabView):

    tab_group_class = project_tabs.PluginDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ plugin.name|default:plugin.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)

        plugin = self.get_data()
        context["plugin"] = plugin
        context["url"] = reverse(self.redirect_url)
        context["actions"] = self._get_actions(plugin)

        return context

    def _get_actions(self, plugin):
        table = project_tables.PluginsTable(self.request)
        return table.render_row_actions(plugin)

    @memoized.memoized_method
    def get_data(self):
        plugin_id = self.kwargs['plugin_id']
        try:
            plugin = iotronic.plugin_get(self.request, plugin_id, None)
        except Exception:
            msg = ('Unable to retrieve plugin %s information') % {'name':
                                                                  plugin.name}
            exceptions.handle(self.request, msg, ignore=True)
        return plugin

    def get_tabs(self, request, *args, **kwargs):
        plugin = self.get_data()
        return self.tab_group_class(request, plugin=plugin, **kwargs)


class PluginDetailView(DetailView):
    redirect_url = 'horizon:iot:plugins:index'

    def _get_actions(self, plugin):

        # Deserialize the code of the plugin
        plugin.code = cPickle.loads(str(plugin.code))

        table = project_tables.PluginsTable(self.request)
        return table.render_row_actions(plugin)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django import template
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from openstack_dashboard import api

LOG = logging.getLogger(__name__)


class CreateBoardLink(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Board")
    url = "horizon:iot:boards:create"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:create_board"),)


class EditBoardLink(tables.LinkAction):
    name = "edit"
    verbose_name = _("Edit")
    url = "horizon:iot:boards:update"
    classes = ("ajax-modal",)
    icon = "pencil"
    # policy_rules = (("iot", "iot:update_board"),)

    """
    def allowed(self, request, role):
        return api.keystone.keystone_can_edit_role()
    """


class RestoreServices(tables.BatchAction):
    name = "restoreservices"

    @staticmethod
    def action_present(count):
        return u"Restore ALL Services"

    @staticmethod
    def action_past(count):
        return u"Restored ALL Services"

    def allowed(self, request, board=None):
        return True

    def action(self, request, board_id):
        api.iotronic.restore_services(request, board_id)


class EnableServiceLink(tables.LinkAction):
    name = "enableservice"
    verbose_name = _("Enable Service(s)")
    url = "horizon:iot:boards:enableservice"
    classes = ("ajax-modal",)
    # icon = "plus"
    # policy_rules = (("iot", "iot:service_action"),)


class DisableServiceLink(tables.LinkAction):
    name = "disableservice"
    verbose_name = _("Disable Service(s)")
    url = "horizon:iot:boards:disableservice"
    classes = ("ajax-modal",)
    # icon = "plus"
    # policy_rules = (("iot", "iot:service_action"),)


class RemovePluginsLink(tables.LinkAction):
    name = "removeplugins"
    verbose_name = _("Remove Plugin(s)")
    url = "horizon:iot:boards:removeplugins"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:remove_plugins"),)


class AttachPortLink(tables.LinkAction):
    name = "attachport"
    verbose_name = _("Attach Port")
    url = "horizon:iot:boards:attachport"
    classes = ("ajax-modal",)
    icon = "plus"


class DetachPortLink(tables.LinkAction):
    name = "detachport"
    verbose_name = _("Detach Port")
    url = "horizon:iot:boards:detachport"
    classes = ("ajax-modal",)
    icon = "plus"


class EnableWebServiceLink(tables.LinkAction):
    name = "enablewebservice"
    verbose_name = _("Enable Web Services Manager")
    url = "horizon:iot:boards:enablewebservice"
    classes = ("ajax-modal",)
    icon = "plus"


class DisableWebServiceLink(tables.LinkAction):
    name = "disablewebservice"
    verbose_name = _("Disable Web Services Manager")
    url = "horizon:iot:boards:disablewebservice"
    classes = ("ajax-modal",)
    icon = "plus"


class DeleteBoardsAction(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Board",
            u"Delete Boards",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Board",
            u"Deleted Boards",
            count
        )
    # policy_rules = (("iot", "iot:delete_board"),)

    """
    def allowed(self, request, role):
        return api.keystone.keystone_can_edit_role()
    """

    def delete(self, request, board_id):
        api.iotronic.board_delete(request, board_id)


class BoardFilterAction(tables.FilterAction):

    # If uncommented it will appear the select menu list of fields
    # and filter button
    """
    filter_type = "server"
    filter_choices = (("name", _("Board Name ="), True),
                      ("type", _("Type ="), True),
                      ("status", _("Status ="), True))
    """

    def filter(self, table, boards, filter_string):
        """Naive case-insensitive search."""
        q = filter_string.lower()
        return [board for board in boards
                if q in board.name.lower()]


def show_services(board_info):
    template_name = 'iot/boards/_cell_services.html'
    context = board_info._info
    # LOG.debug("CONTEXT: %s", context)
    return template.loader.render_to_string(template_name,
                                            context)


class BoardsTable(tables.DataTable):
    name = tables.WrappingColumn('name', link="horizon:iot:boards:detail",
                                 verbose_name=_('Board Name'))
    type = tables.Column('type', verbose_name=_('Type'))
    # mobile = tables.Column('mobile', verbose_name=_('Mobile'))
    lr_version = tables.Column('lr_version', verbose_name=_('LR version'))
    # fleet = tables.Column('fleet', verbose_name=_('Fleet ID'))
    fleet_name = tables.Column('fleet_name', verbose_name=_('Fleet Name'))
    # code = tables.Column('code', verbose_name=_('Code'))
    status = tables.Column('status', verbose_name=_('Status'))
    uuid = tables.Column('uuid', verbose_name=_('Board ID'))
    # location = tables.Column('location', verbose_name=_('Geo'))
    services = tables.Column(show_services, verbose_name=_('Services'))
    # extra = tables.Column('extra', verbose_name=_('Extra'))

    # Overriding get_object_id method because in IoT service the "id" is
    # identified by the field UUID
    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "boards"
        verbose_name = _("boards")
        row_actions = (EditBoardLink, EnableServiceLink, DisableServiceLink,
                       RestoreServices, AttachPortLink, DetachPortLink,
                       EnableWebServiceLink, DisableWebServiceLink,
                       RemovePluginsLink, DeleteBoardsAction)
        table_actions = (BoardFilterAction, CreateBoardLink,
                         DeleteBoardsAction)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django import template
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from openstack_dashboard import api

LOG = logging.getLogger(__name__)


class ExposeWebserviceLink(tables.LinkAction):
    name = "expose"
    verbose_name = _("Expose")
    url = "horizon:iot:webservices:expose"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:expose_webservice"),)


class UnexposeWebserviceLink(tables.LinkAction):
    name = "unexpose"
    verbose_name = _("Unexpose")
    url = "horizon:iot:webservices:unexpose"
    classes = ("ajax-modal",)
    icon = "plus"
    # policy_rules = (("iot", "iot:unexpose_webservice"),)


"""
class DisableWebservicesAction(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Disable",
            u"Disable Web Services",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Disabled Web Service",
            u"Disabled Web Services",
            count
        )
    # policy_rules = (("iot", "iot:disable_webservice"),)

    def delete(self, request, board_id):
        api.iotronic.webservice_disable(request, board_id)
"""


class WebserviceFilterAction(tables.FilterAction):

    def filter(self, table, webservices, filter_string):
        # Naive case-insensitive search.
        q = filter_string.lower()
        return [webservice for webservice in webservices
                if q in webservice.name.lower()]


def show_webservices(board_info):
    template_name = 'iot/webservices/_cell_webservices.html'
    context = board_info._info
    # LOG.debug("CONTEXT: %s", context)
    return template.loader.render_to_string(template_name,
                                            context)


class WebservicesTable(tables.DataTable):

    """
    uuid = tables.WrappingColumn('uuid',
                                 link="horizon:iot:webservices:detail",
                                 verbose_name=_('UUID'))
    """
    board = tables.Column('name', verbose_name=_('Board Name'))

    board_uuid = tables.Column('board_uuid', 
                               verbose_name=_('Board UUID'),
                               hidden=True)

    webservices = tables.Column(show_webservices,
                                verbose_name=_('Web Services'))

    http = tables.Column('http_port', verbose_name=_('HTTP'))
    https = tables.Column('https_port', verbose_name=_('HTTPS'))

    # Overriding get_object_id method because in IoT webservice the "id" is
    # identified by the field UUID
    def get_object_id(self, datum):
        # LOG.debug('SELF: %s', self)
        return datum.board_uuid

    class Meta(object):
        name = "webservices"
        verbose_name = _("Web Services")
        """
        row_actions = (ExposeWebserviceLink,
                       UnexposeWebserviceLink,
                       DisableWebservicesAction)
        table_actions = (WebserviceFilterAction, DisableWebservicesAction)
        """
        row_actions = (ExposeWebserviceLink,
                       UnexposeWebserviceLink)
        table_actions = (WebserviceFilterAction,)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import cPickle
import json
import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
# START FROM HERE !!!!!!! openstack_dashboard/api/nova.py
# from iotronicclient.common.apiclient import exceptions as iot_exceptions

from horizon import forms
from horizon import messages

from openstack_dashboard.api import iotronic
from openstack_dashboard import policy

LOG = logging.getLogger(__name__)


class CreatePluginForm(forms.SelfHandlingForm):
    name = forms.CharField(label=_("Plugin Name"))

    """
    public = forms.ChoiceField(
        label=_("Public"),
        choices =[('false', _('False')), ('true', _('True'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'slug-public'},
        )
    )
    """

    public = forms.BooleanField(label=_("Public"), required=False)
    callable = forms.BooleanField(label=_("Callable"), required=False)

    code = forms.CharField(
        label=_("Code"),
        widget=forms.Textarea(
            attrs={'class': 'switchable', 'data-slug': 'slug-code'})
    )

    parameters = forms.CharField(
        label=_("Parameters"),
        required=False,
        widget=forms.Textarea(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-parameters-create'}),
        help_text=_("Plugin parameters")
    )

    def handle(self, request, data):

        if not data["parameters"]:
            data["parameters"] = {}
        else:
            data["parameters"] = json.loads(data["parameters"])

        try:
            iotronic.plugin_create(request, data["name"],
                                   data["public"], data["callable"],
                                   data["code"], data["parameters"])

            messages.success(request, _("Plugin created successfully."))

            return True
        # except iot_exceptions.ClientException:
        except Exception:
            # LOG.debug("API REQ EXC: %s", request)
            # LOG.debug("API REQ (DICT): %s", exceptions.__dict__)
            exceptions.handle(request, _('Unable to create plugin.'))


class InjectPluginForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Plugin Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    onboot = forms.BooleanField(label=_("On Boot"), required=False)

    board_list = forms.MultipleChoiceField(
        label=_("Boards List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-inject-plugin'}),
        help_text=_("Select boards in this pool ")
    )

    def __init__(self, *args, **kwargs):

        super(InjectPluginForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})

        boardslist_length = len(kwargs["initial"]["board_list"])

        self.fields["board_list"].choices = kwargs["initial"]["board_list"]
        self.fields["board_list"].max_length = boardslist_length

    def handle(self, request, data):

        counter = 0

        for board in data["board_list"]:
            for key, value in self.fields["board_list"].choices:
                if key == board:

                    try:
                        inject = iotronic.plugin_inject(request, key,
                                                        data["uuid"],
                                                        data["onboot"])
                        # LOG.debug("API: %s %s", plugin, request)
                        message_text = inject
                        messages.success(request, _(message_text))

                        if counter != len(data["board_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to inject plugin on board " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break


class StartPluginForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Plugin Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    board_list = forms.MultipleChoiceField(
        label=_("Boards List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-start-boards'}),
        help_text=_("Select boards in this pool ")
    )

    parameters = forms.CharField(
        label=_("Parameters"),
        required=False,
        widget=forms.Textarea(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-startplugin-json'}),
        help_text=_("Plugin parameters")
    )

    def __init__(self, *args, **kwargs):

        super(StartPluginForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})

        boardslist_length = len(kwargs["initial"]["board_list"])

        self.fields["board_list"].choices = kwargs["initial"]["board_list"]
        self.fields["board_list"].max_length = boardslist_length

    def handle(self, request, data):

        counter = 0

        if not data["parameters"]:
            data["parameters"] = {}
        else:
            data["parameters"] = json.loads(data["parameters"])

        for board in data["board_list"]:
            for key, value in self.fields["board_list"].choices:
                if key == board:

                    try:
                        action = iotronic.plugin_action(request, key,
                                                        data["uuid"],
                                                        "PluginStart",
                                                        data["parameters"])
                        # LOG.debug("API: %s %s", plugin, request)
                        message_text = action
                        messages.success(request, _(message_text))

                        if counter != len(data["board_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to start plugin on board " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break


class StopPluginForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Plugin Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    delay = forms.IntegerField(
        label=_("Delay in secs"),
        required=False,
        help_text=_("OPTIONAL: seconds to wait before stopping the plugin")
    )

    board_list = forms.MultipleChoiceField(
        label=_("Boards List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-stop-boards'}),
        help_text=_("Select boards in this pool ")
    )

    def __init__(self, *args, **kwargs):

        super(StopPluginForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})

        boardslist_length = len(kwargs["initial"]["board_list"])

        self.fields["board_list"].choices = kwargs["initial"]["board_list"]
        self.fields["board_list"].max_length = boardslist_length

    def handle(self, request, data):

        counter = 0

        if not data["delay"]:
            data["delay"] = {}
        else:
            data["delay"] = {"delay": data["delay"]}

        for board in data["board_list"]:
            for key, value in self.fields["board_list"].choices:
                if key == board:

                    try:
                        action = iotronic.plugin_action(request, key,
                                                        data["uuid"],
                                                        "PluginStop",
                                                        data["delay"])
                        # LOG.debug("API: %s %s", plugin, request)
                        message_text = action
                        messages.success(request, _(message_text))

                        if counter != len(data["board_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to stop plugin on board " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break


class CallPluginForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Plugin Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    board_list = forms.MultipleChoiceField(
        label=_("Boards List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-call-boards'}),
        help_text=_("Select boards in this pool ")
    )

    parameters = forms.CharField(
        label=_("Parameters"),
        required=False,
        widget=forms.Textarea(
            attrs={'class': 'switchable',
                   'data-slug': 'slug-callplugin-json'}),
        help_text=_("Plugin parameters")
    )

    def __init__(self, *args, **kwargs):

        super(CallPluginForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})

        boardslist_length = len(kwargs["initial"]["board_list"])

        self.fields["board_list"].choices = kwargs["initial"]["board_list"]
        self.fields["board_list"].max_length = boardslist_length

    def handle(self, request, data):

        counter = 0

        if not data["parameters"]:
            data["parameters"] = {}
        else:
            data["parameters"] = json.loads(data["parameters"])

        for board in data["board_list"]:
            for key, value in self.fields["board_list"].choices:
                if key == board:

                    try:
                        action = iotronic.plugin_action(request, key,
                                                        data["uuid"],
                                                        "PluginCall",
                                                        data["parameters"])

                        message_text = action
                        messages.success(request, _(message_text))

                        if counter != len(data["board_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to call plugin on board " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break


class RemovePluginForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Plugin Name'),
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )

    board_list = forms.MultipleChoiceField(
        label=_("Boards List"),
        widget=forms.SelectMultiple(
            attrs={'class': 'switchable', 'data-slug': 'slug-remove-boards'}),
        help_text=_("Select boards in this pool ")
    )

    def __init__(self, *args, **kwargs):

        super(RemovePluginForm, self).__init__(*args, **kwargs)
        # input=kwargs.get('initial',{})

        boardslist_length = len(kwargs["initial"]["board_list"])

        self.fields["board_list"].choices = kwargs["initial"]["board_list"]
        self.fields["board_list"].max_length = boardslist_length

    def handle(self, request, data):

        counter = 0

        for board in data["board_list"]:
            for key, value in self.fields["board_list"].choices:
                if key == board:

                    try:
                        iotronic.plugin_remove(request,
                                               key,
                                               data["uuid"])
                        # LOG.debug("API: %s %s", plugin, request)
                        message_text = "Plugin removed successfully from" \
                                       + " board " + str(value) + "."
                        messages.success(request, _(message_text))

                        if counter != len(data["board_list"]) - 1:
                            counter += 1
                        else:
                            return True

                    except Exception:
                        message_text = "Unable to remove plugin from board " \
                                       + str(value) + "."
                        exceptions.handle(request, _(message_text))

                    break


class UpdatePluginForm(forms.SelfHandlingForm):

    uuid = forms.CharField(label=_("Plugin ID"), widget=forms.HiddenInput)
    owner = forms.CharField(label=_("Owner"), widget=forms.HiddenInput)
    name = forms.CharField(label=_("Plugin Name"))
    public = forms.BooleanField(label=_("Public"), required=False)
    callable = forms.BooleanField(label=_("Callable"), required=False)

    code = forms.CharField(
        label=_("Code"),
        widget=forms.Textarea(
            attrs={'class': 'switchable', 'data-slug': 'slug-code'})
    )

    def __init__(self, *args, **kwargs):

        super(UpdatePluginForm, self).__init__(*args, **kwargs)

        # Admin
        if policy.check((("iot", "iot:update_plugins"),), self.request):
            # LOG.debug("ADMIN")
            pass

        # Admin_iot_project
        elif policy.check((("iot", "iot:update_project_plugins"),),
                          self.request):
            # LOG.debug("IOT ADMIN")

            if self.request.user.id != kwargs["initial"]["owner"]:
                # LOG.debug("NO-edit IOT ADMIN")
                self.fields["name"].widget.attrs = {'readonly': 'readonly'}
                self.fields["public"].widget.attrs = {'disabled': 'disabled'}
                self.fields["callable"].widget.attrs = {'disabled': 'disabled'}
                self.fields["code"].widget.attrs = {'readonly': 'readonly'}

        # Other users
        else:
            if self.request.user.id != kwargs["initial"]["owner"]:
                # LOG.debug("IMMUTABLE FIELDS")
                self.fields["name"].widget.attrs = {'readonly': 'readonly'}
                self.fields["public"].widget.attrs = {'disabled': 'disabled'}
                self.fields["callable"].widget.attrs = {'disabled': 'disabled'}
                self.fields["code"].widget.attrs = {'readonly': 'readonly'}

    def handle(self, request, data):
        try:

            data["code"] = cPickle.dumps(str(data["code"]))

            iotronic.plugin_update(request, data["uuid"],
                                   {"name": data["name"],
                                    "public": data["public"],
                                    "callable": data["callable"],
                                    "code": data["code"]})

            messages.success(request, _("Plugin " + str(data["name"]) +
                                        " updated successfully."))
            return True

        except Exception:
            exceptions.handle(request, _('Unable to update plugin.'))
